﻿------------------------------------------------------------------------------------------
--Frame Control Functions-----------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))

local collectgarbage = collectgarbage
local IsAddOnLoaded = IsAddOnLoaded
local pairs = pairs
local ReloadUI = ReloadUI
local unpack = unpack
local wipe = wipe

function SN:SNCreateBG(frame, type)
	
	--Make the frames look like ElvUI
	if(IsAddOnLoaded("ElvUI")) then
		local E, L, V, P, G,_ = unpack(ElvUI)
		local S = E:GetModule('Skins')
		
		if type == "Frame" then
			frame:StripTextures(true)
			frame:SetTemplate("Transparent")
		elseif type == "Button" then
			S:HandleButton(frame, true)
		elseif type == "EditBox" then
			S:HandleEditBox(frame)
		elseif type == "ScrollBar" then
			S:HandleScrollBar(frame)
		elseif type == "StatusBar" then
			local bg = CreateFrame("Frame")
			bg:SetTemplate("Transparent")
			bg:SetParent(frame)
			bg:ClearAllPoints()
			bg:Point("TOPLEFT", frame, "TOPLEFT", -1, 1)
			bg:Point("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 1, -1)
			bg:SetFrameStrata("BACKGROUND")
			bg:Show()
			bg:SetAlpha(.7)
		end
	end
end

function SN:UnlockFrame(frame)
	if frame == nil then return end

	if SN.data.profile.frames.lock and frame ~= nil then
		if frame == F.powermover then
			F.container:Hide()
		end
		if frame == F.classpower then
			if F.classpower.frame ~= nil then
				F.classpower.frame:Hide()
			end
		end
		
		frame:SetBackdrop( { bgFile   = "Interface/Tooltips/UI-Tooltip-Background",
							edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
							tile     = false,
							tileSize = 0,
							edgeSize = 2,
							insets = { left = 0, right = 0, top = 0, bottom = 0 }
						} )
		frame:SetBackdropColor(.1, .1, .1, .8)
		frame:SetBackdropBorderColor(.1, .1, .1, .5)
			
		frame.value:SetText(frame.name)

		frame.d = frame:CreateTexture("ARTWORK")
		frame.d:SetHeight(16)
		frame.d:SetWidth(16)
		frame.d:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -1, 1)
		frame.d:SetTexture(M.media.draghandle)
		frame.d:SetAlpha(.3)

		frame.tr = frame:CreateTitleRegion()
		frame.tr:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		frame.tr:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, 0)
		frame.tr:SetHeight(21)

		frame:EnableMouse(true)
		frame:SetMovable(true)
		frame:RegisterForDrag("LeftButton")
	end
end

function SN:LockFrame(frame)
	if frame == nil then return end

	if frame == F.powermover then
		if SN.data.profile.powerbar.enable then
			F.container:Show()
		end
	end
	if frame == F.classpower then
		if F.classpower.frame ~= nil then
			F.classpower.frame:Show()
		end
	end
	if not SN.data.profile.frames.lock then
		frame:SetBackdrop(nil)
		frame.d:Hide()
		frame.d = nil
		frame.tr = nil
		frame.value:SetText("")
		frame:EnableMouse(false)
		frame:SetMovable(false)
		
		local a, b, c, d, e = frame:GetPoint()

		if frame == F.powermover then
			SN.data.profile.frames.powerbar = { a, b, c, d, e }
		elseif frame == F.classpower then
			SN.data.profile.frames.classpower = { a, b, c, d, e }
		elseif frame == F.dispel then
			SN.data.profile.frames.dispel = { a, b, c, d, e }
		end
	end
end

function SN:MoveFrames(settings)
	local frames = { F.powermover, F.classpower, F.dispel }

	if settings then
		if frames[1] and F.powermover:IsShown() then
			SN:LockFrame(frames[1])
		end
		if frames[2] and F.classpower:IsShown() then
			SN:LockFrame(frames[2])
		end
		SN:LockFrame(frames[3])
		F.cooldowns:Lock(settings)
		F.debuffs:Lock(settings)

		local dp, dr, drP, dOx, dOy = F.debuffs:GetPoint()
		local cp, cr, crP, cOx, cOy = F.cooldowns:GetPoint()

		if(IsAddOnLoaded("ElvUI")) then
			SN.data.profile.frames.elvdebuffs = { dp, dr, drP, dOx, dOy }
			SN.data.profile.frames.elvcooldowns = { cp, cr, crP, cOx, cOy }
		else
			SN.data.profile.frames.debuffs = { dp, dr, drP, dOx, dOy }
			SN.data.profile.frames.cooldowns = { cp, cr, crP, cOx, cOy }
		end

		SN.data.profile.frames.lock = true
	else
		InterfaceOptionsFrame:Hide()
		if frames[1] and F.powermover:IsShown() then
			SN:UnlockFrame(frames[1])
		end
		if frames[2] and F.classpower:IsShown() then
			SN:UnlockFrame(frames[2])
		end
		SN:UnlockFrame(frames[3])
		F.cooldowns:Lock(settings)
		F.debuffs:Lock(settings)
		
		SN.data.profile.frames.lock = false
	end
	collectgarbage("collect")
end

function SN:ElvOverride()
	local E, L, V, P, G, _ = unpack(ElvUI)
	local UF = E:GetModule("UnitFrames")

	if SN.data.profile.powerbar.enable then
		E.db.unitframe.units.player.power.enable = false
	else
		E.db.unitframe.units.player.power.enable = true
	end
	if SN.data.profile.classResources.enable then
		E.db.unitframe.units.player.classbar.enable = false
		if SN.playerClass == "DRUID" or "ROGUE" then
			E.db.unitframe.units.target.combobar.enable = false
		end
	else
		E.db.unitframe.units.player.classbar.enable = true
		if SN.playerClass == "DRUID" or "ROGUE" then
			E.db.unitframe.units.target.combobar.enable = true
		end
	end
	UF:CreateAndUpdateUF("player")
	UF:CreateAndUpdateUF("target")
	collectgarbage("collect")
end

function SN:ResetAnchors()
	local static = SN.DB.profile.frames
	local temp = SN.data.profile.frames
	local frames = { F.powermover, F.classpower, F.cooldowns, F.dispel }

	for i, frame in pairs(frames) do
		if frame then
			frame:ClearAllPoints()
		end
	end

	if (IsAddOnLoaded("ElvUI")) then
		F.cooldowns:SetPoint(unpack(static.elvcooldowns))
		wipe(temp.elvcooldowns)
	else
		F.cooldowns:SetPoint(unpack(static.cooldowns))
		wipe(temp.cooldowns)
	end
	if not F.classpower then
		F.classpower:SetPoint(unpack(static.classpower))
	end
	if not F.dispel then
		F.dispel:SetPoint(unpack(static.dispel))
	end
	if not F.powermover then
		F.powermover:SetPoint(unpack(static.powerbar))
	end
	wipe(temp.powerbar)
	wipe(temp.dispel)
	wipe(temp.classpower)
	collectgarbage("collect")
	ReloadUI()
end