﻿------------------------------------------------------------------------------------------
--Warlock Resources-----------------------------------------------------------------------
------------------------------------------------------------------------------------------
local SN, L, F, T, M, _ = unpack(select(2,...))
local LSM = LibStub("LibSharedMedia-3.0")

local BORDER = 2
local MAX_POWER_PER_EMBER = 10
local SPELL_POWER_DEMONIC_FURY = SPELL_POWER_DEMONIC_FURY
local SPELL_POWER_BURNING_EMBERS = SPELL_POWER_BURNING_EMBERS
local SPELL_POWER_SOUL_SHARDS = SPELL_POWER_SOUL_SHARDS
local SPEC_WARLOCK_DESTRUCTION = SPEC_WARLOCK_DESTRUCTION
local SPEC_WARLOCK_AFFLICTION = SPEC_WARLOCK_AFFLICTION
local SPEC_WARLOCK_DEMONOLOGY = SPEC_WARLOCK_DEMONOLOGY
local maxShard = SN["classMaxResourceBar"]["WARLOCK"]

function SN:CreateShards()
	local CLASSBAR_WIDTH = SN.data.profile.classResources.width - (BORDER*2)
	
	local shards = CreateFrame("Frame", nil, frame)
	shards:SetSize(CLASSBAR_WIDTH, SN.data.profile.classResources.height)
	shards:SetPoint("CENTER", frame, "CENTER", 0, 0)

	if (IsAddOnLoaded("ElvUI")) then
		SN:SNCreateBG(shards, "Frame")
	else
		shards:SetBackdrop({
			bgFile = M.media.blankTex, 
			edgeFile = M.media.blankTex, 
			tile = false, tileSize = 0, edgeSize = 2, 
			insets = { left = 2, right = 2, top = 2, bottom = 2 }
			})
		shards:SetBackdropColor(0.1, 0.1, 0.1, 1)
		shards:SetBackdropBorderColor(0.1, 0.1, 0.1, 1)
	end

	for i = 1, maxShard do
		shards[i] = CreateFrame("StatusBar", nil, shards)
		shards[i]:SetSize((SN.data.profile.classResources.width - 16) / maxShard, SN.data.profile.classResources.height - 6)
		if i == 1 then
			shards[i]:SetPoint("LEFT", shards, 3, 0)
		else
			shards[i]:SetPoint("LEFT", shards[(i - 1)], "RIGHT", 3, 0)
		end
		shards[i]:SetStatusBarTexture(LSM:Fetch("statusbar", SN.data.profile.classResources.texture))
		shards[i]:GetStatusBarTexture():SetHorizTile(false)
		shards[i]:SetBackdrop({	bgFile = M.media.blankTex })
	end
	
	return shards

end

function SN:SetupShards()
	if SN.playerClass ~= "WARLOCK" then return end
	local frame = F.classpower
	frame.shards = SN:CreateShards(frame)
	F.classpower.frame[#frame.frame + 1] = frame.shards

	frame.shards:RegisterEvent("UNIT_POWER")
	frame.shards:RegisterEvent("PLAYER_TALENT_UPDATE")
	frame.shards:RegisterEvent("PLAYER_ENTERING_WORLD")
	frame.shards:RegisterEvent("UNIT_DISPLAYPOWER")
	frame.shards:SetScript("OnEvent", function(self, event, ...)
		SN:ShardEvents(self)
	end)
	SN:ShardEvents(frame.shards)
end

function SN:ShardEvents(self)
	local color = M.classResources.WARLOCK
	
	local spec = GetSpecialization()
	if spec then
		if not self:IsShown() then 
			self:Show()
		end

		for i = 1, maxShard do
			self[i]:Show()
			self[i]:SetStatusBarColor(color[spec][1], color[spec][2], color[spec][3], 1)
			self[i]:SetBackdropColor(color[spec][1], color[spec][2], color[spec][3], 0.2)
		end
		
		if (spec == SPEC_WARLOCK_DESTRUCTION) then	
			local maxPower = UnitPowerMax("player", SPELL_POWER_BURNING_EMBERS, true)
			local power = UnitPower("player", SPELL_POWER_BURNING_EMBERS, true)
			local numEmbers = power / MAX_POWER_PER_EMBER
			local numBars = floor(maxPower / MAX_POWER_PER_EMBER)
			self.number = numBars
			-- bar unavailable
			
			if numBars == 3 then
				self[4]:Hide()
			else
				self[4]:Show()
			end

			for i = 1, numBars do
				self[i]:SetSize((SN.data.profile.classResources.width - 20) / numBars, SN.data.profile.classResources.height - 6)
				if i == 1 then
					self[i]:SetPoint("LEFT", self, 4, 0)
				else
					self[i]:SetPoint("LEFT", self[(i - 1)], "RIGHT", 4, 0)
				end

				self[i]:SetMinMaxValues((MAX_POWER_PER_EMBER * i) - MAX_POWER_PER_EMBER, MAX_POWER_PER_EMBER * i)
				self[i]:SetValue(power)
			end
		elseif ( spec == SPEC_WARLOCK_AFFLICTION ) then
			local numShards = UnitPower("player", SPELL_POWER_SOUL_SHARDS)
			local maxShards = UnitPowerMax("player", SPELL_POWER_SOUL_SHARDS)
			self.number = maxShards
			
			self[4]:Hide()

			for i = 1, maxShards do
				
				self[i]:SetSize((SN.data.profile.classResources.width - 20) / maxShards, SN.data.profile.classResources.height - 6)
				self[i]:SetMinMaxValues(0, 1)
				if i == 1 then
					self[i]:SetPoint("LEFT", self, 4, 0)
				else
					self[i]:SetPoint("LEFT", self[(i - 1)], "RIGHT", 4, 0)
				end
				if i <= numShards then
					self[i]:SetValue(1)
				else
					self[i]:SetValue(0)
				end
			end
		elseif spec == SPEC_WARLOCK_DEMONOLOGY then
			local power = UnitPower("player", SPELL_POWER_DEMONIC_FURY)
			local maxPower = UnitPowerMax("player", SPELL_POWER_DEMONIC_FURY)
			
			self.number = 1
			self[2]:Hide()
			self[3]:Hide()
			self[4]:Hide()

			self[1]:SetSize((SN.data.profile.classResources.width - 11), SN.data.profile.classResources.height - 6)
			self[1]:SetMinMaxValues(0, maxPower)
			self[1]:SetValue(power)
		end
	else
		if self:IsShown() then 
			self:Hide()
		end
	end
end

function SN:ShardsEnable()
	local frame = F.classpower
	frame.shards:Show()
	frame.shards:RegisterEvent('UNIT_POWER')
	frame.shards:RegisterEvent("PLAYER_TALENT_UPDATE")
	frame.shards:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function SN:ShardsDisable()
	
	local frame = F.classpower
	frame.shards:UnregisterEvent('UNIT_POWER')
	frame.shards:UnregisterEvent("PLAYER_TALENT_UPDATE")
	frame.shards:UnregisterEvent("PLAYER_ENTERING_WORLD")
	frame.shards:Hide()
	collectgarbage("collect")
end